@@ -35,14 +35,15 @@
 #include "dirname.h"
 
 #include "gettext.h"
-#include <locale.h>
 #define _(String) gettext (String)
 #define N_(String) gettext_noop (String)
 
 #include "manconfig.h"
 
+#include "cleanup.h"
 #include "error.h"
 #include "pipeline.h"
+#include "security.h"
 
 #include "descriptions.h"
 #include "ult_src.h"
@@ -135,26 +136,24 @@ int main (int argc, char **argv)
 	program_name = base_name (argv[0]);
 
 	init_debug ();
-
-	if (!setlocale (LC_ALL, "") && !getenv ("MAN_NO_LOCALE_WARNING"))
-		/* Obviously can't translate this. */
-		error (0, 0, "can't set the locale; make sure $LC_* and $LANG "
-			     "are correct");
-	setenv ("MAN_NO_LOCALE_WARNING", "1", 1);
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	bindtextdomain (PACKAGE "-gnulib", LOCALEDIR);
-	textdomain (PACKAGE);
+	pipeline_install_post_fork (pop_all_cleanups);
+	init_locale (LC_ALL, "");
 
 	if (argp_parse (&argp, argc, argv, 0, 0, 0))
 		exit (FAIL);
 
+#ifdef SECURE_MAN_UID
+	/* We aren't setuid, but this allows generic code in lexgrog.l to
+	 * use drop_effective_privs/regain_effective_privs.
+	 */
+	init_security ();
+#endif /* SECURE_MAN_UID */
+
 	if (parse_man)
 		type = 0;
 	else
 		type = 1;
 
-	pipeline_install_sigchld ();
-
 	for (i = 0; i < num_files; ++i) {
 		lexgrog lg;
 		const char *file;
@@ -207,6 +206,8 @@ int main (int argc, char **argv)
 				printf ("\n");
 			}
 			free_descriptions (descs);
+			free (lg.filters);
+			free (lg.whatis);
 		}
 
 		if (!found) {
